-- debug_status = 1
debug_mod_name = "Drones"
debug_file = debug_mod_name .. "-debug.txt"
require("util")
require("utils")
require("config")

local nb_drones_per_page_init = 10

--------------------------------------------------------------------------------------
function init_globals()	
	-- initialize or update general globals of the mod
	debug_print( "init_globals " )

	global.ticks = global.ticks or 0
	global.player_mem = global.player_mem or {}
	global.antennas = global.antennas or {}
	if global.refresh == nil then global.refresh = false end
end

--------------------------------------------------------------------------------------
local function init_player( player )
	if global.player_mem == nil then return end
	
	-- original variables
	
	local player_mem =
	{ 
		character = nil, -- record player main character
		gui_opened = false,
		pagecur = 1, -- current page (1 .. pagenb)
		pagenb = 1, -- total number of pages
		nb_drones_per_page = nb_drones_per_page_init,
		drone_cur = nil, -- current drone if droning
		drone_ren = nil, -- drone being renamed
		drones = {}, -- list of drones = {entity, name, is_maint, cover, dist}
		force = player.force, 
		color = player.color,
	}
	
	global.player_mem[player.index] = global.player_mem[player.index] or player_mem

	-- additionnal late variables 
	player_mem = global.player_mem[player.index]

	if player.connected and player.character and player.character.name == "player" then player_mem.character = player.character end	
	player_mem.tick_last_creation = player_mem.tick_last_creation or 0
	player_mem.drone_del_confirm = player_mem.drone_del_confirm or nil
	player_mem.drone_del_confirm_tick = player_mem.drone_del_confirm_tick or 0
	player_mem.no_drone = player_mem.no_drone or 0
	player_mem.pointer = player_mem.pointer or nil
	player_mem.drone_cur_cover = player_mem.drone_cur_cover or -1
	player_mem.gui_dist = player_mem.gui_dist or nil
	
	if player.connected then
		if player.force.technologies["drones-surveillance"].researched then
			if player.gui.top.but_drones == nil then
				player.gui.top.add({type = "sprite-button", name = "but_drones", sprite = "sprite_drones_main", style = "drones_sprite_main_style"})	
			end
		else
			if player.gui.top.but_drones then player.gui.top.but_drones.destroy() end
		end
	end
end

--------------------------------------------------------------------------------------
local function init_players()
	for _, player in pairs(game.players) do
		init_player(player)
	end
end

--------------------------------------------------------------------------------------
local function on_init() 
	-- called once, the first time the mod is loaded on a game (new or existing game)
	debug_print( "on_init" )
	init_globals()
	init_players()
end

script.on_init(on_init)

--------------------------------------------------------------------------------------
local function on_configuration_changed(data)
	-- detect any mod or game version change
	if data.mod_changes ~= nil then
		local changes = data.mod_changes[debug_mod_name]
		if changes ~= nil then
			debug_print( "update mod: ", debug_mod_name, " ", tostring(changes.old_version), " to ", tostring(changes.new_version) )
		
			if global.players_mem ~= nil then
				global.player_mem = table.deepcopy(global.players_mem)
				global.players_mem = nil
			end
			
			init_globals()
			
			for _,force in pairs(game.forces) do
				force.recipes["drone-antenna"].enabled = force.technologies["drones-surveillance"].researched
			end
			
			-- migrations
			
			if changes.old_version and older_version(changes.old_version , "1.0.14") then
				for _,player in pairs(game.players) do
					local player_mem = global.player_mem[player.index]
					if player_mem.pointer == true then 
						player.clear_gui_arrow()
						player_mem.pointer = nil
					end
					if player.gui.top.but_drones ~= nil then player.gui.top.but_drones.destroy() end -- new sprite button
				end
			end
			
			if data.mod_changes["CameraDrones"] ~= nil and older_version(data.mod_changes["CameraDrones"].old_version, "1.0.4") then
				debug_print( "migrate mod: CameraDrones" )
				local nb_surv, nb_maint
				nb_surv=0
				nb_maint=0
				local surf = game.surfaces["nauvis"]
				
				for _, ent in pairs(surf.find_entities_filtered({name= "camera-drone"})) do
					ent.destroy()
					nb_surv=nb_surv+1
				end
				for _, ent in pairs(surf.find_entities_filtered({name= "drone-surveillance"})) do
					ent.destroy()
					nb_surv=nb_surv+1
				end
				for _, ent in pairs(surf.find_entities_filtered({name= "camera-drone-maint"})) do
					ent.destroy()
					nb_maint=nb_maint+1
				end
				for _, ent in pairs(surf.find_entities_filtered({name= "drone-maintenance"})) do
					ent.destroy()
					nb_maint=nb_maint+1
				end
				
				for _,player in pairs(game.players) do
					player.print( "Give back " .. nb_surv .. "+" .. nb_maint .. " drones." )
					if nb_surv > 0 then player.insert( {name = "drone-surveillance", count = nb_surv} ) end
					if nb_maint > 0 then player.insert( {name = "drone-maintenance", count = nb_maint} ) end
					if nb_surv+nb_maint > 0 then player.force.technologies["drones-surveillance"].researched = true end
				end	
			end
			
			init_players()
			
			if changes.old_version and older_version(changes.old_version , "1.0.18") then
				message_all( "Drones : now you can use drones in other surfaces !" )
			end
		end
	end
end

script.on_configuration_changed(on_configuration_changed)

--------------------------------------------------------------------------------------
local function on_research_finished(event)
	if event.research ~= nil and (event.research.name == "drones-surveillance" or event.research.name == "drones-maintenance") then
		init_players()
	end
end

script.on_event(defines.events.on_research_finished, on_research_finished )

--------------------------------------------------------------------------------------
local function on_creation( event )
	local ent = event.created_entity
	
	debug_print( "on_creation" )
	
	if ent.name == "drone-surveillance" or ent.name == "drone-maintenance" then
		if event.player_index ~= nil then
			local player = game.players[event.player_index]
			
			if player.character.name == "player" then
				add_drone( player, ent, false )
			else
				player.print({"gui-drones-cannot-drone"})
				ent.destroy()
			end
			
			global.refresh = true
		else
			ent.destroy()
		end
		
	elseif ent.name == "drone-antenna" then
		table.insert( global.antennas, ent )
		global.refresh = true
		
	elseif ent.type == "electric-pole" then
		global.refresh = true
	end
end

script.on_event(defines.events.on_built_entity, on_creation )
script.on_event(defines.events.on_robot_built_entity, on_creation )

--------------------------------------------------------------------------------------
local function on_destruction( event )
	local ent = event.entity

	debug_print( "on_destruction" )
	
	if ent.name == "drone-antenna" then
		for k, ant in pairs(global.antennas) do
			if ant == ent then 
				table.remove(global.antennas, k)
				break
			end
		end
		global.refresh = true
		
	elseif ent.type == "electric-pole" then
		global.refresh = true
	end
end

script.on_event(defines.events.on_entity_died, on_destruction )
script.on_event(defines.events.on_robot_pre_mined, on_destruction )
script.on_event(defines.events.on_preplayer_mined_item, on_destruction )

--------------------------------------------------------------------------------------
local function on_player_created(event)
	debug_print( "player created" )
	local player = game.players[event.player_index]
	init_player( player )
	
	if debug_status == 1 or is_dev(player) then
		player.insert({name = "drone-surveillance", count = 10} )
		player.insert({name = "drone-maintenance", count = 5} )
		player.insert({name = "drone-antenna", count = 5} )
	end
end

script.on_event(defines.events.on_player_created, on_player_created )

--------------------------------------------------------------------------------------
local function on_player_joined_game(event)
	-- called in SP(once) and MP(every connect), eventually after on_player_created
	local player = game.players[event.player_index]
	debug_print( "player joined ", player.name )
	
	init_player(player)
end

script.on_event(defines.events.on_player_joined_game, on_player_joined_game )

--------------------------------------------------------------------------------------
local function on_pre_player_died(event)
	local player = game.players[event.player_index]
	debug_print( "player dying ", player.name )
end

script.on_event(defines.events.on_pre_player_died, on_pre_player_died )

--------------------------------------------------------------------------------------
local function on_player_died(event)
	local player = game.players[event.player_index]
	debug_print( "player died ", player.name )
end

script.on_event(defines.events.on_player_died, on_player_died )

--------------------------------------------------------------------------------------
local function on_player_respawned(event)
	local player = game.players[event.player_index]
	debug_print( "player respawned ", player.name )
	
	local player_mem = global.player_mem[player.index]
	player_mem.character = player.character
end

script.on_event(defines.events.on_player_respawned, on_player_respawned )

--------------------------------------------------------------------------------------
local function on_player_changed_surface(event)
	local player = game.players[event.player_index]
	debug_print( "on_player_changed_surface ", player.name )
	
	message_all( {"gui-drones-warning-surface"} )
	local player_mem = global.player_mem[player.index]
	if player_mem and player.character then
		if player_mem.drone_cur then -- in drone
			player_mem.drone_cur.entity = player.character
		else
			player_mem.character = player.character
		end
	end
end

script.on_event(defines.events.on_player_changed_surface, on_player_changed_surface )

--------------------------------------------------------------------------------------
local function on_tick(event)
	if global.ticks <= 0 then
		global.ticks = 124
	else
		global.ticks = global.ticks - 1
	end
	
	if global.ticks == 119 then
		-- test if body dead while in drone
		for _,player in pairs(game.players) do
			if player.connected then
				local player_mem = global.player_mem[player.index]
				if player_mem and player_mem.drone_cur then -- in drone
					if not (player_mem.character.valid and player_mem.character.name == "player") then 
						player.print( {"gui-drones-body-died"} )
						-- create new body to sacrifice
						local character = player.surface.create_entity({name = "player", position = player.position, force = player.force})
						if character then
							player.character = character -- reintegrate new body
							player_mem.character = character
							player_mem.drone_cur.entity.active = false -- in case moving when body die
							player_mem.drone_cur = nil
							player_mem.drone_cur_cover = -1
							player.character.die() -- kill new body
						end
					end
				end
			end
		end
		
	elseif global.ticks == 78 then
		for _,player in pairs(game.players) do
			if player.connected then
				-- test if del confirm out of time
				local player_mem = global.player_mem[player.index]
				if player_mem.drone_del_confirm then
					if game.tick > player_mem.drone_del_confirm_tick then
						player_mem.drone_del_confirm = nil
						if player.gui.left.flow_drones then update_menu_drones(player) end
					end
				end
				-- test if refresh needed
				if global.refresh and player.gui.left.flow_drones then 
					debug_print("refresh")
					calc_antennas_ranges()
					update_menu_drones(player)
				end
			end
		end
		
		global.refresh = false
		
	elseif global.ticks == 37 then
		for _,player in pairs(game.players) do
			if player.connected and player.character then
				-- test if item in drone slots, and put it back in toolbelt
				if player.character.name == "drone-surveillance" then
					check_empty_inventory(player,defines.inventory.player_tools)
					check_empty_inventory(player,defines.inventory.player_guns)
					check_empty_inventory(player,defines.inventory.player_ammo)
					check_empty_inventory(player,defines.inventory.player_armor)
				elseif player.character.name == "drone-maintenance" then
					check_empty_inventory(player,defines.inventory.player_guns)
					check_empty_inventory(player,defines.inventory.player_ammo)
					check_empty_inventory(player,defines.inventory.player_armor)
				end
			end
		end
		
	elseif global.ticks%30 == 0 then
		for _,player in pairs(game.players) do
			if player.connected then
				-- test if drone in antenna range
				check_antenna_range(player)
			end
		end
		
	end
end

script.on_event(defines.events.on_tick, on_tick)

--------------------------------------------------------------------------------------
function drone_flying_text(player,cover)
	if cover == 2 then
		player.surface.create_entity({name = "flying-text", position = pos_offset(player.position,{x=-0.5,y=0.2}), text = {"gui-drones-flying-ok"}, color = {r=0, g=1, b=0}})
	elseif cover == 1 then
		player.print( {"gui-drones-antenna-warning"} )
		player.surface.create_entity({name = "flying-text", position = pos_offset(player.position,{x=-0.5,y=0.2}), text = {"gui-drones-flying-warning"}, color = {r=1, g=1, b=0}})
	else
		player.print( {"gui-drones-antenna-lost"} )
		player.surface.create_entity({name = "flying-text", position = pos_offset(player.position,{x=-0.5,y=0.2}), text = {"gui-drones-flying-lost"}, color = {r=1, g=0, b=0}})
	end
end

--------------------------------------------------------------------------------------
function check_antenna_range(player)
	local player_mem = global.player_mem[player.index]
	local changes = false
	
	if player_mem.drone_cur and player_mem.drone_cur.entity and player_mem.drone_cur.entity.valid then
		for _, drone in pairs( player_mem.drones ) do
			if calc_antenna_range(drone) then
				changes = true
				local cover = drone.cover
				
				if drone == player_mem.drone_cur and cover ~= player_mem.drone_cur_cover then
					player_mem.drone_cur_cover = cover

					if cover == 2 then
						player_mem.drone_cur.entity.active = true
					elseif cover == 1 then
						player_mem.drone_cur.entity.active = true
					else
						player_mem.drone_cur.entity.active = false
					end
					
					drone_flying_text(player,cover)
				end
			end
		end	
	end
	
	if changes then
		update_menu_drones(player)				
	end
	
	if player_mem.drone_cur and player_mem.gui_dist then
		player_mem.gui_dist.caption = "@" .. math.floor(player_mem.drone_cur.dist) .. "m"
	end
end

--------------------------------------------------------------------------------------
function calc_antenna_range(drone)
	-- return true if cover changed

	if drone == nil or drone.entity == nil or not drone.entity.valid then 
		drone.cover = 2
		return(true) 
	end
	
	local pos = drone.entity.position
	local surf = drone.entity.surface
	local force = drone.entity.force
	local distMin = 0
	local result = 0 
	
	-- for _, ant in pairs(global.antennas) do
	for i=#global.antennas,1,-1 do
		ant = global.antennas[i]
		if ant and ant.valid then
			if ant.force == force and ant.surface == surf and ant.energy > 0 then
				local dist = distance_square( ant.position, pos )
				if distMin == 0 then
					distMin = dist
				else
					distMin = math.min(distMin,dist)
				end
				if dist <= drone_antenna_radius_warning then
					result = 2
					break
				elseif dist <= drone_antenna_radius_offline then
					result = 1
				end
			end
		else
			table.remove(global.antennas,i)
		end
	end
	
	drone.dist = distMin
	
	if drone.cover == result then return( false ) end
	
	drone.cover = result
	return( true )
end		

--------------------------------------------------------------------------------------
function calc_antennas_ranges()
	for _, player in pairs(game.players) do
		if player.connected then
			local player_mem = global.player_mem[player.index]
			for _, drone in pairs( player_mem.drones ) do
				calc_antenna_range(drone)
			end
		end
	end
end

--------------------------------------------------------------------------------------
local function on_gui_click(event)
	debug_print( "player index=", event.player_index, " event=", event.element.name )
	
	if event.element.name == "but_drones" then
		-- main menu toggle
		local player = game.players[event.player_index]
		local player_mem = global.player_mem[player.index]
		
		if player.gui.left.flow_drones then
			player.gui.left.flow_drones.destroy()
			player_mem.gui_dist = nil
		else
			update_menu_drones(player)
		end
		
	elseif event.element.name == "but_drone_pagedown" then
		local player = game.players[event.player_index]
		local player_mem = global.player_mem[player.index]
		
		player_mem.pagecur = math.max( 1, player_mem.pagecur - 1 )
		update_menu_drones(player)
		
	elseif event.element.name == "but_drone_pageup" then
		local player = game.players[event.player_index]
		local player_mem = global.player_mem[player.index]
		
		player_mem.pagecur = math.min( player_mem.pagenb, player_mem.pagecur + 1 )
		update_menu_drones(player)

	elseif string.sub(event.element.name, 1, 14) == "but_drone_del_" then
		-- delete drone
		local player = game.players[event.player_index]
		local player_mem = global.player_mem[player.index]
		local n_drone = tonumber(string.sub(event.element.name, 15))
		local drone_del = player_mem.drones[n_drone]
		
		if drone_del then
			if drone_del.entity and drone_del.entity.valid then
				if player.character.name ~= "player" or player_mem.drone_cur ~= nil then
					player.print({"gui-drones-cannot-delete-inside"})
				elseif player.surface ~= drone_del.entity.surface then
					player.print({"gui-drones-cannot-delete-far"})
				elseif distance_square( player.position, drone_del.entity.position ) > drone_recover_radius then
					player.print({"gui-drones-cannot-delete-far"})
				elseif player_mem.drone_del_confirm == drone_del then
					local drone_name = drone_del.entity.name
					
					player_mem.drone_cur = nil
					player_mem.drone_cur_cover = -1
					
					if prepare_del_drone(player,drone_del.entity) then
						if player.can_insert(drone_name) then
							for i, drone in pairs(player_mem.drones) do
								if drone == drone_del then
									table.remove(player_mem.drones,i)
									break
								end
							end
							
							drone_del.entity.destroy()
							player.insert({name=drone_name, count=1})
						else
							player.print( {"gui-drones-cannot-delete-space"} )
						end
					else
						player.print( {"gui-drones-cannot-delete-space"} )
					end
				else
					player.print( {"gui-drones-confirm-delete"} )
					player_mem.drone_del_confirm = drone_del
					player_mem.drone_del_confirm_tick = game.tick + 180
				end
			else
				for i, drone in pairs(player_mem.drones) do
					if drone == drone_del then
						table.remove(player_mem.drones,i)
						break
					end
				end
			end
			
			update_menu_drones(player)
		end
		
	elseif string.sub(event.element.name, 1, 14) == "but_drone_ren_" then
		-- rename drone
		local player = game.players[event.player_index]
		local player_mem = global.player_mem[player.index]
		local n_drone = tonumber(string.sub(event.element.name, 15))
		local drone_ren = player_mem.drones[n_drone]
		
		if drone_ren then
			player_mem.drone_ren = drone_ren
			show_rename(player, drone_ren)
			
			update_menu_drones(player)
		end
		
	elseif event.element.name == "but_drone_body" then
		-- switch to body
		local player = game.players[event.player_index]
		local player_mem = global.player_mem[player.index]

		if player.character.active and player.walking_state.walking then
			player.print( {"gui-drones-stop-moving"} )
		else
			-- back to body
			if player_mem.drone_cur then 
				player_mem.drone_cur.entity = player.character
				switch_to_body( player, player_mem )
				player_mem.drone_cur = nil
				player_mem.drone_cur_cover = -1
				update_menu_drones(player)
			end
		end

	elseif string.sub(event.element.name, 1, 14) == "but_drone_swi_" then
		-- switch from man to drones
		local player = game.players[event.player_index]
		local player_mem = global.player_mem[player.index]
		local n_drone = tonumber(string.sub(event.element.name, 15))
		local drone_new = player_mem.drones[n_drone]
		calc_antenna_range(drone_new)
		
		debug_print("sel=",n_drone)

		if player.character.active and player.walking_state.walking then
			player.print( {"gui-drones-stop-moving"} )
		else
			if drone_new then
				local drone_cur = player_mem.drone_cur
				
				if drone_new ~= drone_cur then
					-- to another drone
					if drone_cur then
						player_mem.drone_cur.entity = player.character
					end
					if switch_to_drone( player, player_mem, drone_new ) then
						player_mem.drone_cur = drone_new
					end
				else
					-- back to body
					if drone_cur then 
						player_mem.drone_cur.entity = player.character
						switch_to_body( player, player_mem )
						player_mem.drone_cur = nil
						player_mem.drone_cur_cover = -1
					end
				end
				
				update_menu_drones(player)
			end
		end

	elseif string.sub(event.element.name, 1, 14) == "but_drone_ptr_" then
		-- point to drone
		local player = game.players[event.player_index]
		local player_mem = global.player_mem[player.index]
		local n_drone = tonumber(string.sub(event.element.name, 15))
		local drone = player_mem.drones[n_drone]
		
		if drone and drone.entity and drone.entity.valid then
			clear_arrow(player,player_mem)
			
			if player_mem.pointer == drone.entity then
				player_mem.pointer = nil
			else
				if drone.entity.surface == player.surface then
					player.set_gui_arrow( { type = "entity", entity = drone.entity } )
					player_mem.pointer = drone.entity
				end
			end
		end

	elseif event.element.name == "but_drone_rename_ok" then
		local player = game.players[event.player_index]
		local player_mem = global.player_mem[player.index]
		
		if player.gui.center.frame_drone_rename ~= nil then
			local s = player.gui.center.frame_drone_rename.textfield_drone_rename.text
			if s == "" then s = "???" end
			player_mem.drone_ren.name = s
			player.gui.center.frame_drone_rename.destroy()
			update_menu_drones(player)
		end
		
		player_mem.drone_ren = nil
		
	elseif event.element.name == "but_drone_rename_cancel" then
		local player = game.players[event.player_index]
		local player_mem = global.player_mem[player.index]
		
		if player.gui.center.frame_drone_rename ~= nil then
			player.gui.center.frame_drone_rename.destroy()
		end
		
		player_mem.drone_ren = nil
	end
end

script.on_event(defines.events.on_gui_click, on_gui_click )

--------------------------------------------------------------------------------------
function check_empty_inventory( player, inv_type )
	local inv_weapon = player.get_inventory(inv_type)
	if not inv_weapon.is_empty() then
		player.print( {"gui-drones-not-wear-this"} )
		for i = 1, #inv_weapon do
			local stack = inv_weapon[i]
			if stack.valid_for_read then 
				local cursor = player.cursor_stack
				if cursor and cursor.valid_for_read then
					player.get_inventory(defines.inventory.player_quickbar).insert(stack)
				else
					player.cursor_stack.set_stack(stack)
				end
				inv_weapon.remove(stack)
			end
		end
	end
end

--------------------------------------------------------------------------------------
function add_drone( player, ent, nocheck )
	local player_mem = global.player_mem[player.index]
	
	if nocheck or game.tick > player_mem.tick_last_creation + 15 then
		-- if ent.name == "drone-surveillance" then ent.destructible = false end
		ent.destructible = false
		
		if player ~= nil and #player_mem.drones < 100 then
			local drone_new
			player_mem.no_drone = player_mem.no_drone + 1
			if ent.name == "drone-maintenance" then
				drone_new = { entity = ent, name = "Drone #" .. player_mem.no_drone, is_maint = true, cover = -1, dist = 0 }
			else
				drone_new = { entity = ent, name = "Drone #" .. player_mem.no_drone, is_maint = false, cover = -1, dist = 0 }
			end
			table.insert( player_mem.drones, drone_new )
			player_mem.tick_last_creation = game.tick
			calc_antenna_range(drone_new)
			update_menu_drones(player)
			return(drone_new)
		end
	else
		player.insert( {name = ent.name, count = 1 } )
		ent.destroy()
	end
	
	return( nil )
end

--------------------------------------------------------------------------------------
function prepare_del_drone( player, ent )
	local result = true
	result = result and move_inventory( player, ent.get_inventory(defines.inventory.player_main) )
	result = result and move_inventory( player, ent.get_inventory(defines.inventory.player_quickbar) )
	result = result and move_inventory( player, ent.get_inventory(defines.inventory.player_guns) )
	result = result and move_inventory( player, ent.get_inventory(defines.inventory.player_ammo) )
	result = result and move_inventory( player, ent.get_inventory(defines.inventory.player_armor) )
	result = result and move_inventory( player, ent.get_inventory(defines.inventory.player_tools) )
	result = result and move_inventory( player, ent.get_inventory(defines.inventory.player_trash) )
	return(result)
end

--------------------------------------------------------------------------------------
function move_inventory( player, inv )
	local ok = true
	local contents = inv.get_contents()
	
	for name, nb in pairs(contents) do
		player.print( name .. "=" .. nb )
		local result = player.insert({name = name, count = nb})
		if result > 0 then
			inv.remove({name = name, count = result})
		end
		if result < nb then
			ok = false
		end
	end
	
	return( ok )
end

--------------------------------------------------------------------------------------
function update_menu_drones(player)
	local guif = player.gui.left.flow_drones
	local player_mem = global.player_mem[player.index]
	
	if player.gui.left.flow_drones then
		player.gui.left.flow_drones.destroy()
	end
	
	if #player_mem.drones > 0 then
		local guif2, but

		player_mem.pagenb = 1 + math.floor( (#player_mem.drones-1) / player_mem.nb_drones_per_page )
		
		guif = player.gui.left.add({type = "flow", name = "flow_drones", direction = "vertical", style = "drones_flow_style"})
		guif2 = guif.add({type = "frame", name = "flow_body", direction = "horizontal", style = "drones_frame_style"})
		guif2 = guif2.add({type = "flow", direction = "horizontal", style = "drones_flow_style"})
		guif2.add({type = "button", name = "but_drone_pagedown", caption = "<", style = "drones_button_style", tooltip={"gui-drones-paged-tt"}})
		guif2.add({type = "button", name = "but_drone_pagecur", caption = player_mem.pagecur .. "/" .. player_mem.pagenb, style = "drones_button_style", tooltip={"gui-drones-page-tt"}})
		guif2.add({type = "button", name = "but_drone_pageup", caption = ">", style = "drones_button_style", tooltip={"gui-drones-pageu-tt"}})
		but = guif2.add({type = "button", name = "but_drone_body", caption = {"gui-drones-body"}, style = "drones_button_style", tooltip={"gui-drones-body-tt"}})
		if player_mem.drone_cur == nil then but.style.font_color = colors.green end
		
		player_mem.gui_dist = nil
		
		for i, drone in pairs(player_mem.drones) do
			if i > player_mem.pagecur * player_mem.nb_drones_per_page then break end
			if i >= 1 + (player_mem.pagecur-1) * player_mem.nb_drones_per_page then
				guif2 = guif.add({type = "frame", name = "flow_drone_" .. i, direction = "horizontal", style = "drones_frame_style"})
				guif2 = guif2.add({type = "flow", direction = "horizontal", style = "drones_flow_style"})
				
				but = guif2.add({type = "button", name = "but_drone_del_" .. i, caption = "X", style = "drones_button_style", tooltip={"gui-drones-delete-tt"}})
				if drone == player_mem.drone_del_confirm then but.style.font_color = colors.red end
				but = guif2.add({type = "button", name = "but_drone_ren_" .. i, caption = "N", style = "drones_button_style", tooltip={"gui-drones-rename-tt"}})
				if drone == player_mem.drone_del_confirm then but.style.font_color = colors.red end
				but = guif2.add({type = "button", name = "but_drone_ptr_" .. i, caption = "?", style = "drones_button_style", tooltip={"gui-drones-find-tt"}})
				but = guif2.add({type = "button", name = "but_drone_swi_" .. i, caption = drone.name, style = "drones_button_style", tooltip={"gui-drones-switch-tt"}})
				if drone.is_maint == true then but.style.font_color = colors.orange end
				if drone == player_mem.drone_cur then but.style.font_color = colors.green end
				if drone == player_mem.drone_del_confirm then but.style.font_color = colors.red end
				if not drone.entity.valid then but.style.font_color = colors.purple end
				but = guif2.add({type = "label", name = "lab_drone_elc_" .. i, caption = "@" .. math.floor(drone.dist) .. "m", style = "drones_label_style", tooltip={"gui-drones-distance-tt"}})
				if drone.cover == 2 then
					-- but.caption = "hi"
					but.style.font_color = colors.green
				elseif drone.cover == 1 then
					-- but.caption = "lo"
					but.style.font_color = colors.yellow
				else
					-- but.caption = "!!"
					but.style.font_color = colors.red
				end
				if drone == player_mem.drone_cur then player_mem.gui_dist = but end
			end
		end
	end
	
	return(guif)
end

--------------------------------------------------------------------------------------
function show_rename(player,drone)
	local guif = player.gui.center.frame_drone_rename
	local player_mem = global.player_mem[player.index]
	
	if guif == nil then
		guif = player.gui.center.add({type = "frame", name = "frame_drone_rename", caption = {"gui-drones-rename"}, direction = "horizontal", style = "drones_frame_style"})
		guif.add({type = "textfield", name = "textfield_drone_rename", text = " ", style = "drones_textfield_style"})
		guif.add({type = "button", name = "but_drone_rename_ok", caption = "OK", style = "drones_button_style"})
		guif.add({type = "button", name = "but_drone_rename_cancel", caption = {"gui-drones-rename-cancel"}, style = "drones_button_style"})
	end
	
	guif.textfield_drone_rename.text = drone.name
end

--------------------------------------------------------------------------------------
function clear_arrow(player,player_mem)
	if player_mem.pointer then
		player.clear_gui_arrow()
	end
end

--------------------------------------------------------------------------------------
function refresh_arrow(player,player_mem)
	if player_mem.pointer then
		if player_mem.pointer.valid then
			player.set_gui_arrow( { type = "entity", entity = player_mem.pointer } )
		else
			player_mem.pointer = nil
		end
	end
end

--------------------------------------------------------------------------------------
function switch_to_char( player, char )
	player.character = nil
	player.teleport(char.position,char.surface)
	player.character = char
end

--------------------------------------------------------------------------------------
function switch_to_drone( player, player_mem, drone_new )
	-- character to drone
	local character = player.character
	
	if character == nil or (not character.valid) then
		player.print( {"gui-drones-wait-respawn"} )
		
	elseif character.name == "player" then
		local target = drone_new.entity
		-- debug_print( "man to drone... ", target.name )
		if target and target.valid then
			debug_print( "target ok..." )
			clear_arrow(player,player_mem)
			player.surface.create_entity({name = "flying-text", position = pos_offset(player.position,{x=-0.5,y=0.2}), text = {"gui-drones-flying-droning"}, color = {r=1, g=0, b=0}})
			switch_to_char( player, target )
			target.active = (drone_new.cover > 0)
			drone_flying_text(player,drone_new.cover)
			-- refresh_arrow(player,player_mem)
			return( drone_new )
		end
		
		return( nil )
		
	elseif character.name == "drone-surveillance" or character.name == "drone-maintenance" then
		debug_print( "drone to drone..." )
		local target = drone_new.entity
		if target and target.valid then
			clear_arrow(player,player_mem)
			switch_to_char( player, target )
			target.active = (drone_new.cover > 0)
			drone_flying_text(player,drone_new.cover)
			-- refresh_arrow(player,player_mem)
			return( drone_new )
		end
		
		return( nil )
	else
		player.print( {"gui-drones-not-in-player-drone"} )
	end
	
	return( nil )
end

--------------------------------------------------------------------------------------
function switch_to_body( player, player_mem )
	local body = player_mem.character
	
	if body and body.valid and body.name == "player" then -- if body alive (death while droning detected in on_tick)
		debug_print( "back in body..." )
		clear_arrow(player,player_mem)
		switch_to_char( player, body )
		player.surface.create_entity({name = "flying-text", position = pos_offset(player.position,{x=-0.5,y=0.2}), text = {"gui-drones-flying-back"}, color = {r=0, g=1, b=0}})
		-- refresh_arrow(player,player_mem)
	end
end

--------------------------------------------------------------------------------------
function find_lost_character(force)
	for _, surf in pairs(game.surfaces) do
		local chars = surf.find_entities_filtered({name= "player",force=force})
		if chars then return( chars[1] ) end
	end
	return(nil)
end


--------------------------------------------------------------------------------------
local interface = {}

function interface.reset()
	debug_print( "reset" )
	
	for _, player in pairs(game.players) do
		global.player_mem[player.index] = nil
		init_player(player)
	end
	
	init_globals()
end

function interface.rebuild(player)
	debug_print( "rebuild" )
	
	if player == nil then player = game.players[1] end
	
	if player.character and player.character.name == "player" then
		local player_mem = global.player_mem[player.index]
		
		player_mem.drones = {}
		local ents
		
		for _,surf in pairs(game.surfaces) do
			ents = surf.find_entities_filtered({name = "drone-maintenance",force=player.force})
			for _, ent in pairs(ents) do
				add_drone(player,ent,true)
			end
			ents = surf.find_entities_filtered({name = "drone-surveillance",force=player.force})
			for _, ent in pairs(ents) do
				add_drone(player,ent,true)
			end
		end
		
		-- update_menu_drones(player)
		-- calc_antennas_ranges()
	end
end

function interface.lost(player)
	debug_print( "lost" )
	
	if player == nil then player = game.players[1] end
	local player_mem = global.player_mem[player.index]
	
	-- if player_mem.character == nil then
		lost = find_lost_character(player.force)
	-- else
		-- lost = player_mem.character
	-- end
	
	if lost then
		player.character = lost
		player_mem.drone_cur = nil
		player_mem.drone_cur_cover = -1
		update_menu_drones(player)
	end
end

function interface.freedrones()
	debug_print( "freedrones" )

	for _,force in pairs(game.forces) do
		force.technologies["drones-surveillance"].researched = true
		force.technologies["drones-maintenance"].researched = true
	end
	
	for _, player in pairs(game.players) do
		player.insert({name = "drone-surveillance", count = 10} )
		player.insert({name = "drone-maintenance", count = 5} )
		player.insert({name = "drone-antenna", count = 5} )
	end
end

remote.add_interface( "drones", interface )

-- /c remote.call( "drones", "reset" )
-- /c remote.call( "drones", "reset" )
-- /c remote.call( "drones", "rebuild", game.player )
-- /c remote.call( "drones", "freedrones" )
